@@ -3,8 +3,8 @@
 ;; Copyright (C) 2023  Ilya Chernyshov
 
 ;; Author: Ilya Chernyshov <ichernyshovvv@gmail.com>
-;; Package-Version: 0.2
-;; Package-Revision: 830479285fef
+;; Package-Version: 20241027.805
+;; Package-Revision: e61e5734b49f
 ;; Package-Requires: ((emacs "28.1") (compat "29.1.4.1") (org "9.0") (svg "1.1"))
 ;; Keywords: org, calendar, timeblocking, agenda
 ;; URL: https://github.com/ichernyshovvv/org-timeblock
@@ -36,17 +36,126 @@
 
 (require 'org)
 (require 'svg)
-(require 'color)
 (require 'seq)
 (require 'compat)
 (require 'compat-macs)
 
 ;;;; Faces
 
+;; The colors are borrowed from pulsar.el project written by Protesilaos Stavrou
+
+(defface org-timeblock-red
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#ffcccc" :foreground "#77002a")
+    (((class color) (min-colors 88) (background dark))
+     :background "#77002a" :foreground "#ffcccc")
+    (t :inverse-video t))
+  "Red face."
+  :group 'org-timeblock)
+
+(defface org-timeblock-green
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#aceaac" :foreground "#00422a")
+    (((class color) (min-colors 88) (background dark))
+     :background "#00422a" :foreground "#aceaac")
+    (t :inverse-video t))
+  "Green face."
+  :group 'org-timeblock)
+
+(defface org-timeblock-yellow
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#fff29a" :foreground "#693200")
+    (((class color) (min-colors 88) (background dark))
+     :background "#693200" :foreground "#fff29a")
+    (t :inverse-video t))
+  "Yellow face."
+  :group 'org-timeblock)
+
+(defface org-timeblock-blue
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#8fcfff" :foreground "#242679")
+    (((class color) (min-colors 88) (background dark))
+     :background "#242679" :foreground "#8fcfff")
+    (t :inverse-video t))
+  "Blue face."
+  :group 'org-timeblock)
+
+(defface org-timeblock-magenta
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#ffccff" :foreground "#71206a")
+    (((class color) (min-colors 88) (background dark))
+     :background "#71206a" :foreground "#ffccff")
+    (t :inverse-video t))
+  "Magenta face."
+  :group 'org-timeblock)
+
+(defface org-timeblock-cyan
+  '((default :extend t)
+    (((class color) (min-colors 88) (background light))
+     :background "#8eecf4" :foreground "#004065")
+    (((class color) (min-colors 88) (background dark))
+     :background "#004065" :foreground "#8eecf4")
+    (t :inverse-video t))
+  "Cyan face."
+  :group 'org-timeblock)
+
 (defface org-timeblock-list-header '((t (:inherit org-agenda-structure)))
   "Face used in org-timeblock-list for dates."
   :group 'org-timeblock)
 
+(defface org-timeblock-select
+  '((default :extend t)
+    (((class color)
+      (min-colors 88)
+      (background light))
+     :background "#f3d000")
+    (((class color) (min-colors 88) (background dark))
+     :background "#3f1651")
+    (t :inverse-video t))
+  "Face used for selected blocks."
+  :group 'org-timeblock)
+
+(defface org-timeblock-mark
+  '((default :extend t)
+    (((class color)
+      (min-colors 88)
+      (background light))
+     :background "#7b435c")
+    (((class color) (min-colors 88) (background dark))
+     :background "#7b435c")
+    (t :inverse-video t))
+  "Face used for marked blocks."
+  :group 'org-timeblock)
+
+(defface org-timeblock-hours-line
+  '((default :extend t)
+    (((class color)
+      (min-colors 88)
+      (background light))
+     :background "#7b435c")
+    (((class color) (min-colors 88) (background dark))
+     :background "#cdcdcd")
+    (t :inverse-video t))
+  "Face used for hour lines."
+  :group 'org-timeblock)
+
+(defface org-timeblock-current-time-indicator
+  '((default :extend t)
+    (((class color)
+      (min-colors 88)
+      (background light))
+     :background "red")
+    (((class color) (min-colors 88) (background dark))
+     :background "red")
+    (t :inverse-video t))
+  "Color face used for current time indicator."
+  :group 'org-timeblock)
+
 ;;;; Custom Variables
 
 (defgroup org-timeblock nil
@@ -64,8 +173,9 @@ When set to the symbol `next' only the first future repeat is shown."
 	  (const :tag "Do not show repeated entries" nil)))
 
 (defcustom org-timeblock-files 'agenda
-  "Non-nil shows repeated entries in the future dates of repeat.
-When set to the symbol `next' only the first future repeat is shown."
+  "Org files with agenda items to display with `org-timeblock'.
+When set to the symbol \\+`agenda', `org-agenda-files' are used.
+Otherwise, it may be set to a list of filenames."
   :group 'org-timeblock
   :type '(choice
 	  (repeat :tag "List of files" file)
@@ -124,40 +234,27 @@ When set to the symbol `next' only the first future repeat is shown."
 
 (defcustom org-timeblock-tag-colors
   nil
-  "Colors for specific tags.
+  "Faces for specific tags.
 
-List of lists where each list is of the form
-  (\"tagname\" \"background color\" \"foreground color\").
-Colors are set in hex format.  Example:
-
-  ((\"tag1\" \"#f3d000\" \"#000000\")
-   (\"tag2\" \"#ff8f88\" \"#000000\"))
+Alist of the form
+  ((\"tag1\" . face-1)
+   (\"tag2\" . face-2)).
 
 In `org-timeblock-mode', timeblocks tagged with a tag in car are
-painted in background color.  In `org-timeblock-list-mode', both
-background and foreground colors are used to colorize items that
+painted in :background color of the face in cdr.  In
+`org-timeblock-list-mode', faces are used to colorize items that
 are tagged with a tag in car."
   :type 'list
   :group 'org-timeblock)
 
 ;;;; Variables
 
-(defvar org-timeblock-mark-color "#7b435c")
-
 (defvar org-timeblock-markers nil)
 
 (defvar org-timeblock-buffers nil)
 
 (defvar org-timeblock-mark-count 0)
 
-(defvar org-timeblock-select-color-light "#f3d000")
-
-(defvar org-timeblock-select-color-dark "#3f1651")
-
-(defvar org-timeblock-select-color org-timeblock-select-color-light)
-
-(defvar org-timeblock-background-color (face-attribute 'default :background))
-
 (defvar org-timeblock-colors nil)
 
 (defvar org-timeblock-data nil)
@@ -420,6 +517,14 @@ Compare only hours and minutes."
   "Return ITEM's \\='timestamp text property as decoded time."
   (org-timeblock-timestamp-to-time (org-timeblock-get-ts-prop item)))
 
+(defsubst org-timeblock-get-saved-random-face (title)
+  "Get saved random color face for TITLE.
+If not found, generate it with `org-timeblock--random-color',
+save it and return."
+  (or (alist-get title org-timeblock-colors nil nil #'equal)
+      (setf (alist-get title org-timeblock-colors nil nil #'equal)
+	    (org-timeblock--random-color))))
+
 (defun org-timeblock-timestamp< (a b)
   "Return t, if A's \\='timestamp is less then B's."
   (org-timeblock-on org-timeblock-get-ts org-timeblock-time< a b))
@@ -446,7 +551,10 @@ Compare only hours and minutes."
     (org-timeblock-unselect-block)
     (when-let ((node (car (dom-by-id org-timeblock-svg id))))
       (dom-set-attribute node 'orig-fill (dom-attr node 'fill))
-      (dom-set-attribute node 'fill org-timeblock-select-color)
+      (dom-set-attribute
+       node 'fill
+       (face-attribute
+	'org-timeblock-select :background))
       (dom-set-attribute node 'select t)
       (setq org-timeblock-column (dom-attr node 'column)))
     (with-current-buffer org-timeblock-buffer
@@ -471,51 +579,15 @@ Otherwise, return nil.
 	(< y1 y2)
 	(< y2 y1-end))))))
 
-(defun org-timeblock--parse-hex-color (hex)
-  "Convert a HEX color code to a RGB list of form (R G B)."
-  (cl-loop for scale in (color-name-to-rgb hex) collect (* scale 255)))
-
 (defun org-timeblock--random-color ()
-  "Generate random color based on BASE-COLOR and RANGE.
-Default background color is used when BASE-COLOR is nil."
-  (let* ((default-background-color
-	  (ignore-errors (face-attribute 'default :background)))
-	 (base-color
-          (cond ((eq 'unspecified default-background-color)
-                 "#fff")
-		((string-match "^#[0-9a-fA-F]\\{3,6\\}" default-background-color)
-                 default-background-color)
-		((color-name-to-rgb default-background-color) ;; yellow, LightBlue, etc...
-                 default-background-color)
-		(t "#fff")))
-	 (range 50))
-    (when (color-name-to-rgb base-color)
-      (let (rgb (hex "#"))
-	(mapc (lambda (x)
-		(setq rgb (cons (round (* x 255)) rgb)))
-	      (color-name-to-rgb base-color))
-	(setq rgb (nreverse rgb))
-	(mapc (lambda (x)
-		(setq hex (concat hex (format "%02x" x))))
-	      rgb)
-	(setq base-color hex)))
-    (let* ((rgb (org-timeblock--parse-hex-color base-color))
-           (half-range (/ range 2))
-           (fn (lambda (n)
-		 (let ((base (nth n rgb))
-                       (min half-range)
-                       (max (- 255 half-range))
-                       result)
-                   (when (< base min) (setq base min))
-                   (when (> base max) (setq base max))
-                   (setq result (+ (- (cl-random range) half-range) base))
-                   (when (< result 0) (setq result 0))
-                   (when (> result 255) (setq result 255))
-                   result)))
-           (r (funcall fn 0))
-           (g (funcall fn 1))
-           (b (funcall fn 2)))
-      (format "#%02x%02x%02x" r g b))))
+  "Return random org-timeblock color face."
+  (seq-random-elt
+   '(org-timeblock-red
+     org-timeblock-green
+     org-timeblock-yellow
+     org-timeblock-blue
+     org-timeblock-magenta
+     org-timeblock-cyan)))
 
 (defun org-timeblock--timestamp-relevant-p (timestamp date)
   "Check if org-element TIMESTAMP is relevant to DATE.
@@ -625,29 +697,8 @@ DATE is decoded-time value."
 			 (columns
 			  (mapcar (lambda (x) (cons (get-text-property 0 'id x) 1)) entries))
 			 placed
-			 (bg-rgb-sum (apply #'+ (org-timeblock--parse-hex-color org-timeblock-background-color)))
-			 (get-color
-			  (if (string= org-timeblock-background-color
-				       (face-attribute 'default :background))
-			      (lambda (title)
-				(cl-callf (lambda (x) (or x (org-timeblock--random-color)))
-				    (alist-get title org-timeblock-colors nil nil #'equal)))
-			    (setq org-timeblock-background-color
-				  (face-attribute 'default :background))
-			    (setq org-timeblock-select-color
-				  (if (> (setq bg-rgb-sum
-					       (apply #'+
-						      (org-timeblock--parse-hex-color
-						       org-timeblock-background-color)))
-					 550)
-				      org-timeblock-select-color-light
-				    org-timeblock-select-color-dark))
-			    (lambda (title)
-			      (setf (alist-get title org-timeblock-colors
-					       nil nil #'equal)
-				    (org-timeblock--random-color)))))
 			 (hour-lines-color
-			  (if (> bg-rgb-sum 550) "#7b435c" "#cdcdcd")))
+			  (face-attribute 'org-timeblock-hours-line :background nil t)))
 		    (dolist (entry entries)
 		      (let* ((timestamp (org-timeblock-get-ts-prop entry))
 			     (repeated (org-element-property
@@ -791,7 +842,9 @@ DATE is decoded-time value."
 			  cur-time-indicator
 			  (+ column-width (* column-width iter))
 			  cur-time-indicator
-			  :stroke "red"))
+			  :stroke (face-attribute
+				   'org-timeblock-current-time-indicator
+				   :background nil t)))
 		    ;; Drawing all the entries inside the timeline
 		    (dolist (entry entries)
 		      (when-let ((length
@@ -843,8 +896,8 @@ DATE is decoded-time value."
 				    `(,title))))
 			(let ((time-string
 			       (get-text-property 0 'time-string entry))
-			      (colors (org-timeblock-get-colors
-				       (get-text-property 0 'tags entry))))
+			      (face (org-timeblock-get-colors
+				     (get-text-property 0 'tags entry))))
 			  (when (< (/ block-width (default-font-width))
 				   (length time-string))
 			    (setq time-string nil))
@@ -882,9 +935,12 @@ DATE is decoded-time value."
 			   :fill
 			   (or
 			    (and
-			     (eq 'deadline
-				 (org-timeblock-get-ts-type entry)) "#5b0103")
-			    (car colors) (funcall get-color title))
+			     (eq 'deadline (org-timeblock-get-ts-type entry))
+			     "#5b0103")
+			    (and face (face-attribute face :background nil 'default))
+			    (face-attribute
+			     (org-timeblock-get-saved-random-face title)
+			     :background nil 'default))
 			   ;; Same timestamp can be displayed in multiple
 			   ;; columns, so _column-number postfix is used to tell
 			   ;; that blocks apart
@@ -901,11 +957,12 @@ DATE is decoded-time value."
 					:fill
 					(or
 					 (and
-					  (eq 'deadline
-					      (org-timeblock-get-ts-type entry))
+					  (eq 'deadline (org-timeblock-get-ts-type entry))
 					  "#ffffff")
-					 (cadr colors)
-					 (face-attribute 'default :foreground))
+					 (and face (face-attribute face :foreground nil 'default))
+					 (face-attribute
+					  (org-timeblock-get-saved-random-face title)
+					  :foreground nil 'default))
 					:font-size
 					(aref (font-info (face-font 'default)) 2))))
 			  (when time-string
@@ -920,7 +977,8 @@ DATE is decoded-time value."
 					(eq 'deadline
 					    (org-timeblock-get-ts-type entry))
 					"#ffffff")
-				       (cadr colors) hour-lines-color)
+				       (and face (face-attribute face :foreground nil 'default))
+				       hour-lines-color)
 				      :font-size
 				      (aref (font-info (face-font 'default)) 2)))))))
 		(let ((message "No data."))
@@ -982,7 +1040,7 @@ DATE is decoded-time value."
 			     (org-timeblock-format-time date-format (nth iter dates)))
 		     'face
 		     (and (= org-timeblock-column (1+ iter))
-			  `(:background ,org-timeblock-select-color)))))
+			  'org-timeblock-select))))
 		result))))
     (svg-possibly-update-image org-timeblock-svg)))
 
@@ -1166,7 +1224,7 @@ If MARKER is nil, use timestamp at point."
     (with-current-buffer buffer
       (org-with-wide-buffer
        (goto-char (point-min))
-       (while (re-search-forward org-ts-regexp nil t)
+       (while (re-search-forward org-tsr-regexp nil t)
 	 (if (save-match-data
 	       (or (org-entry-is-done-p)
 		   (progn
@@ -1267,6 +1325,8 @@ Return nil if buffers are up-to-date."
       ((buffers-to-update
 	(mapcar
 	 #'find-file-noselect
+	 ;; This code is partially borrowed from `org-ql--select-cached'
+	 ;; function which is part of org-ql project written by Adam Porter
 	 (seq-filter
 	  (lambda (file)
 	    (let* ((buffer (find-file-noselect file))
@@ -1294,8 +1354,7 @@ Return nil if buffers are up-to-date."
     t))
 
 (defun org-timeblock-get-colors (tags)
-  "Return the colors for TAGS.
-Return value is of the form (\"background color\" \"foreground color\")."
+  "Return face for TAGS."
   (catch 'found
     (dolist (tag tags)
       (when-let ((colors (cdr (seq-find (lambda (x) (string= (car x) tag))
@@ -1634,6 +1693,8 @@ SLOT should be specified as a plain symbol, not a keyword."
 (cl-defun org-timeblock-time-apply (time &key second minute hour
 					 day month year)
   "Return new timestamp based on TIME with new slot values from keys."
+  ;; This code is borrowed from `ts-apply' function which is part of ts.el
+  ;; project written by Adam Porter
   (let ((time (copy-sequence time)))
     (when second
       (setf (decoded-time-second time) second))
@@ -1776,7 +1837,7 @@ Duration format:
     (when-let ((block (org-timeblock-selected-block))
 	       (marker (org-timeblock-selected-block-marker)))
       (org-timeblock--schedule-time marker)))
-    (org-timeblock-redraw-buffers))
+  (org-timeblock-redraw-buffers))
 
 (defun org-timeblock-set-duration ()
   "Interactively change SCHEDULED duration of the selected block.
@@ -1813,17 +1874,17 @@ SCHEDULED property."
 
 (defun org-timeblock-list-update-entry ()
   "Update text and text properties for the entry at point in *org-timeblock-list*."
-  (let ((marker (get-text-property (line-beginning-position) 'marker)) colors)
+  (let ((marker (get-text-property (line-beginning-position) 'marker)))
     (unless (marker-buffer marker)
       (user-error "Non-existent marker's buffer"))
     (let* ((inhibit-read-only t)
 	   (type (get-text-property (line-beginning-position) 'type))
+	   tags
 	   (new-entry
 	    (org-with-point-at marker
 	      (let ((timestamp (org-element-timestamp-parser))
-		    (title (org-get-heading t nil t t))
-		    (tags (mapcar #'substring-no-properties (org-get-tags))))
-		(setq colors (org-timeblock-get-colors tags))
+		    (title (org-get-heading t nil t t)))
+		(setq tags (mapcar #'substring-no-properties (org-get-tags)))
 		(propertize
 		 (concat (org-timeblock--construct-prefix timestamp type)
 			 title)
@@ -1837,9 +1898,7 @@ SCHEDULED property."
       (insert (propertize
 	       (concat new-entry "\n")
 	       'face
-	       `(:extend t ,@(and (car colors) (list :background (car colors)))
-			 ,@(and (cadr colors)
-				(list :foreground (cadr colors)))))))))
+	       (org-timeblock-get-colors tags))))))
 
 ;;;; Navigation commands
 
@@ -1863,7 +1922,10 @@ SCHEDULED property."
 			       (> (cdr pos) y))))))))
       (unless (dom-attr node 'mark)
 	(dom-set-attribute node 'orig-fill (dom-attr node 'fill)))
-      (dom-set-attribute node 'fill org-timeblock-select-color)
+      (dom-set-attribute
+       node 'fill
+       (face-attribute
+	'org-timeblock-select :background))
       (dom-set-attribute node 'select t))
     (setq org-timeblock-column
 	  (1+ (/ (car pos) (/ window-width org-timeblock-span))))
@@ -1881,7 +1943,8 @@ Otherwise, return nil."
     (dom-set-attribute
      node
      'fill (if (dom-attr node 'mark)
-	       org-timeblock-mark-color
+	       (face-attribute
+		'org-timeblock-mark :background)
 	     (or (dom-attr node 'orig-fill) "#ffffff")))
     (dom-remove-attribute node 'select)
     (cons (dom-attr node 'order)
@@ -1908,7 +1971,10 @@ Otherwise, return nil."
   (interactive)
   (when-let ((node (org-timeblock-selected-block))
 	     ((not (dom-attr node 'mark))))
-    (dom-set-attribute node 'fill org-timeblock-mark-color)
+    (dom-set-attribute
+     node 'fill
+     (face-attribute
+      'org-timeblock-mark :background))
     (dom-set-attribute node 'mark t)
     (cl-incf org-timeblock-mark-count))
   (org-timeblock-forward-block))
@@ -1926,7 +1992,10 @@ Otherwise, return nil."
     (when-let ((id (get-text-property 0 'id entry))
 	       (node (car (dom-by-id org-timeblock-svg id))))
       (dom-set-attribute node 'orig-fill (dom-attr node 'fill))
-      (dom-set-attribute node 'fill org-timeblock-mark-color)
+      (dom-set-attribute
+       node 'fill
+       (face-attribute
+	'org-timeblock-mark :background))
       (dom-set-attribute node 'mark t)
       (cl-incf org-timeblock-mark-count)))
   (org-timeblock-redisplay))
@@ -1981,7 +2050,10 @@ Otherwise, return nil."
 					   (= (dom-attr node 'order) 0))))))))
     (unless (dom-attr node 'mark)
       (dom-set-attribute node 'orig-fill (dom-attr node 'fill)))
-    (dom-set-attribute node 'fill org-timeblock-select-color)
+    (dom-set-attribute
+     node 'fill
+     (face-attribute
+      'org-timeblock-select :background))
     (dom-set-attribute node 'select t)
     (org-timeblock-show-olp-maybe (org-timeblock-selected-block-marker))
     (org-timeblock-redisplay) t))
@@ -2048,7 +2120,10 @@ Return t on success, otherwise - nil."
 			       (= (dom-attr node 'order) (1- len))))))))))
     (unless (dom-attr node 'mark)
       (dom-set-attribute node 'orig-fill (dom-attr node 'fill)))
-    (dom-set-attribute node 'fill org-timeblock-select-color)
+    (dom-set-attribute
+     node 'fill
+     (face-attribute
+      'org-timeblock-select :background))
     (dom-set-attribute node 'select t)
     (org-timeblock-show-olp-maybe (org-timeblock-selected-block-marker))
     (org-timeblock-redisplay) t))
@@ -2251,15 +2326,11 @@ Available view options:
 	    'face 'org-timeblock-list-header))
 	  (insert "\n")
 	  (dolist (entry entries)
-	    (let ((colors (org-timeblock-get-colors
-			   (get-text-property 0 'tags entry))))
-	      (insert (propertize
-		       (concat entry "\n")
-		       'face
-		       `(:extend t ,@(and (car colors)
-					  (list :background (car colors)))
-				 ,@(and (cadr colors)
-					(list :foreground (cadr colors))))))))
+	    (insert (propertize
+		     (concat entry "\n")
+		     'face
+		     (org-timeblock-get-colors
+		      (get-text-property 0 'tags entry)))))
 	  (goto-char (point-max))))
       (goto-char (point-min))
       (when (get-buffer-window org-timeblock-buffer)
@@ -2279,7 +2350,11 @@ SVG image (.svg), PDF (.pdf) is produced."
 		   (not (y-or-n-p
 			 (format "Overwrite existing file %s? " file))))))
       (user-error "Cannot write agenda to file %s" file))
-  (let ((file (expand-file-name file)))
+  (org-timeblock-unselect-block)
+  (let ((file (expand-file-name file))
+	(svg (copy-sequence org-timeblock-svg)))
+    ;; delete marker to not trigger `svg-possibly-update-image'
+    (dom-remove-attribute svg :image)
     (with-temp-buffer
       (let* ((dates (org-timeblock-get-dates
 		     (car org-timeblock-daterange)
@@ -2293,7 +2368,7 @@ SVG image (.svg), PDF (.pdf) is produced."
 		((pred (< 6)) "[%m-%d]")
 		((pred (< 3)) "[%d]"))))
 	(dom-add-child-before
-	 org-timeblock-svg
+	 svg
 	 (dom-node
 	  'rect
 	  (list
@@ -2304,11 +2379,11 @@ SVG image (.svg), PDF (.pdf) is produced."
 	   (cons 'fill (face-attribute 'default :background)))))
 	(dotimes (iter (length dates))
 	  (svg-text
-	   org-timeblock-svg (org-timeblock-format-time date-format (nth iter dates))
+	   svg (org-timeblock-format-time date-format (nth iter dates))
 	   :y org-timeblock-svg-height
 	   :x (+ 5 (* (/ org-timeblock-svg-width (length dates)) iter))
 	   :fill (face-attribute 'default :foreground)))
-	(svg-print org-timeblock-svg))
+	(svg-print svg))
       (pcase (file-name-extension file)
 	((or "pdf" "png")
 	 (unless (executable-find "inkscape")
